@@ -9,6 +9,12 @@ int main() {
   return -1;
 }
 #else
+#if (_TAKE2 > 0)
+int main() {
+  fprintf(stderr, "Error: tool does not work on big endian\n");
+  return -1;
+}
+#endif
 
 #include <sys/types.h>
 #include <sys/time.h>
@@ -26,7 +32,7 @@ int rawmode = 0, seq = 1, id = 0, num = 0;
 char hash[20] = "", *key = NULL, vec[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
 
 void help(char *prg) {
-  printf("%s %s (c) 2012 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("%s %s (c) 2013 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
   printf("Syntax: %s [-k key] interface file\n\n", prg);
   printf("Options:\n");
   printf("  -k key     decrypt the content with Blowfish-160\n");
