@@ -18,9 +18,9 @@ char *ptr3, *ptr4;
 int i;
 
 void help(char *prg) {
-  printf("%s %s (c) 2012 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("%s %s (c) 2013 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
   printf("Syntax: %s interface [script]\n\n", prg);
-  printf("This tools detects new ipv6 addresses joining the local network.\n");
+  printf("This tools detects new IPv6 addresses joining the local network.\n");
   printf("If script is supplied, it is executed with the detected IPv6 address as first\nand the interface as second command line option.\n\n");
   exit(-1);
 }
