@@ -25,7 +25,7 @@ int main(int argc, char **argv) {
   int sock, pid = getpid(), dlen = 0, i = 0, len;
 
   if (argc != 4) {
-    printf("%s %s (c) 2012 by %s %s\n\n", argv[0], VERSION, AUTHOR, RESOURCE);
+    printf("%s %s (c) 2013 by %s %s\n\n", argv[0], VERSION, AUTHOR, RESOURCE);
     printf("Syntax: %s dns-server full-qualified-host-dns-name ipv6address\n\n", argv[0]);
     printf("Example: %s dns.test.com myhost.sub.test.com ::1\n\n", argv[0]);
     exit(0);
@@ -54,11 +54,11 @@ int main(int argc, char **argv) {
 
   freeaddrinfo(ai);
   if ((dst = thc_resolve6(argv[3])) == NULL) {
-    fprintf(stderr, "Error: not a valid ipv6 address: %s\n", argv[3]);
+    fprintf(stderr, "Error: not a valid IPv6 address: %s\n", argv[3]);
     exit(-1);
   }
 
-  memcpy(buf, (char *) &pid, 2);
+  memcpy(buf, (char *) &pid + _TAKE2, 2);
   memcpy(buf + 2, b1, sizeof(b1));
   i = 2 + sizeof(b1);
 
