@@ -16,7 +16,7 @@ unsigned char *frip6, *frmac;
 thc_ipv6_hdr *frhdr = NULL;
 
 void help(char *prg) {
-  printf("%s %s (c) 2012 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("%s %s (c) 2013 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
   printf("Syntax: %s [-HFD] interface network-address/prefix-length [dns-server [router-ip-link-local [mtu [mac-address]]]]\n\n", prg);
   printf("Announce yourself as a router and try to become the default router.\n");
   printf("If a non-existing link-local or mac address is supplied, this results in a DOS.\n");
@@ -33,7 +33,7 @@ void send_rs_reply(u_char *foo, const struct pcap_pkthdr *header, const unsigned
   if (ipv6hdr[6] != NXT_ICMP6 || ipv6hdr[40] != ICMP6_ROUTERSOL || header->caplen < 14 + 40 + 2)
     return;
 
-  if ((pkt = thc_create_ipv6(frint, PREFER_LINK, &pkt_len, frip6, dst, 255, 0, 0, 0xe0, 0)) == NULL)
+  if ((pkt = thc_create_ipv6_extended(frint, PREFER_LINK, &pkt_len, frip6, dst, 255, 0, 0, 0xe0, 0)) == NULL)
     return;
   if (do_hop) {
     type = NXT_HBH;
@@ -256,7 +256,7 @@ int main(int argc, char *argv[]) {
 
   frbuflen = i;
 
-  if ((pkt = thc_create_ipv6(interface, PREFER_LINK, &pkt_len, ip6, dst, 255, 0, 0, 0xe0, 0)) == NULL)
+  if ((pkt = thc_create_ipv6_extended(interface, PREFER_LINK, &pkt_len, ip6, dst, 255, 0, 0, 0xe0, 0)) == NULL)
     return -1;
 
   if (do_hop) {
