@@ -19,7 +19,7 @@ int type = 0, passive = 0, active = 0;
 char *interface;
 
 void help(char *prg) {
-  printf("%s %s (c) 2012 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("%s %s (c) 2013 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
   printf("Syntax: %s [-ap] interface\n\n", prg);
   printf("This tool answers to keep-alive requests on PPPoE and 6in4 tunnels; for PPPoE\nit also sends keep-alive requests.\n");
   printf("Note that the appropriate environment variable THC_IPV6_{PPPOE|6IN4} must be set\n");
@@ -51,7 +51,7 @@ void intercept(u_char *foo, const struct pcap_pkthdr *header, const unsigned cha
     if (ipv6hdr[40] == ICMP6_PINGREQUEST) {
       printf("Keep-alive ping request ID 0x%x seen\n", htonl(*seq));
       if (passive == 0) {
-        if ((pkt = thc_create_ipv6(interface, PREFER_GLOBAL, &pkt_len, ipv6hdr + 24, ipv6hdr + 8, 255, 0, 0, 0, 0)) == NULL)
+        if ((pkt = thc_create_ipv6_extended(interface, PREFER_GLOBAL, &pkt_len, ipv6hdr + 24, ipv6hdr + 8, 255, 0, 0, 0, 0)) == NULL)
           return;
         if (thc_add_icmp6(pkt, &pkt_len, ICMP6_PINGREPLY, 0, htonl(*seq), (unsigned char *) ipv6hdr + 40 + 8, len - 40 - 8, 0) < 0)
           return;
