@@ -24,27 +24,32 @@ unsigned short int baseport = 1200;
 unsigned int pid = 0;
 unsigned int mtu = 0;
 unsigned int orig_mtu = 0;
-int offset = 48, buf_len = 16, tunnel = 0, do_alert = 0, do_toobig = 0, do_frag = 0, do_dst = 0;
-int up_to, complete = 0, type = 0, rawmode = 0, finaldst = 0;
+int udp = 0, offset = 48, buf_len = 16, tunnel = 0, do_alert = 0, do_reply = 0, do_toobig = 0, do_frag = 0, do_dst = 0, do_dst2 = 0;
+int up_to = MAX_SEND, complete = 0, type = 0, rawmode = 0, finaldst = 0;
 
 void help(char *prg) {
-  printf("%s %s (c) 2012 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
-  printf("Syntax: %s [-abdt] [-s src6] interface targetaddress [port]\n\n", prg);
+  printf("%s %s (c) 2013 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("Syntax: %s [-abdtu] [-s src6] interface targetaddress [port]\n\n", prg);
   printf("Options:\n");
   printf("  -a       insert a hop-by-hop header with router alert option.\n");
+  printf("  -D       insert a destination extension header\n");
+  printf("  -E       insert a destination extension header with an invalid option\n");
+  printf("  -F       insert a one-shot fragmentation header\n");
   printf("  -b       instead of an ICMP6 Ping, use TooBig (you will not see the target)\n");
+  printf("  -B       instead of an ICMP6 Ping, use PingReply (you will not see the target)\n");
   printf("  -d       resolves the IPv6 addresses to DNS.\n");
   printf("  -t       enables tunnel detection\n");
+  printf("  -u       use UDP instead of TCP if a port is supplied\n");
   printf("  -s src6  specifies the source IPv6 address\n");
   printf("Maximum hop reach: %d\n\n", INCREASE * (SENDS - 1) + MAX_SEND);
   printf("A basic but very fast traceroute6 program.\n");
   printf("If no port is specified, ICMP6 Ping requests are used, otherwise TCP SYN\n");
-  printf("packets to the specified port.\n");
+  printf("packets to the specified port. Options D, E and F can be use multiple times.\n");
   exit(-1);
 }
 
 void check_packets(u_char *foo, const struct pcap_pkthdr *header, const unsigned char *data) {
-  int i, ok = 0, len = header->caplen;
+  int i, ok = 0, len = header->caplen, add, add2;
   unsigned char *ptr = (unsigned char *) data, *ptr2;
   unsigned short int *si;
   unsigned int *ui, new_mtu = 0;
@@ -60,6 +65,8 @@ void check_packets(u_char *foo, const struct pcap_pkthdr *header, const unsigned
     if ((ptr[0] & 240) != 0x60)
       return;
   }
+  add = do_alert + do_frag + do_dst;
+  add2 = do_alert + do_dst;
 
   if (debug)
     thc_dump_data(ptr, len, "Received Packet");
@@ -69,11 +76,15 @@ void check_packets(u_char *foo, const struct pcap_pkthdr *header, const unsigned
     new_mtu = (ptr[44] << 24) + (ptr[45] << 16) + (ptr[46] << 8) + ptr[47];
     if (new_mtu < mtu) {
       if (type != 2) {
-        pos = ptr[94 + 8*(do_alert + do_frag + do_dst)];
-        pos2 = ptr[93 + 8*(do_alert + do_frag + do_dst)];
+        pos = ptr[94 + 8 * (add)];
+        pos2 = ptr[93 + 8 * (add)];
+        if (pos != pos2 || pos > up_to) {
+          pos = ptr[94 + 8 * (add2)];
+          pos2 = ptr[93 + 8 * (add2)];
+        }
       } else {
-        pos = ptr[101 + 0x30 + 8*(do_alert + do_frag + do_dst + do_toobig - 1)];
-        pos2 = ptr[100 + 0x30 + 8*(do_alert + do_frag + do_dst + do_toobig - 1)];
+        pos = ptr[101 + 0x30 + 8 * (add + do_toobig - 1)];
+        pos2 = ptr[100 + 0x30 + 8 * (add + do_toobig - 1)];
       }
       if (pos == pos2 && pos <= up_to) {
         rmtu[pos - 1] = new_mtu;
@@ -86,14 +97,10 @@ void check_packets(u_char *foo, const struct pcap_pkthdr *header, const unsigned
   if (type != 1) {
     if (ptr[6] != NXT_ICMP6)
       return;
-    if (type == 0)
-      ptr2 = ptr + 96 + 8*(do_alert + do_frag + do_dst);
-    else
-      ptr2 = ptr + 104 + 0x30 + 8*(do_alert + do_frag + do_dst);
-//printf("type:%d  %d %d %d %d, %d %d\n", type, ptr2[0], ptr2[1], ptr2[2], ptr2[3], buf2[0], buf2[1]);
-    if (memcmp(ptr2, buf2, 4) != 0)     // from a different process?
-      return;
     if (ptr[40] == ICMP6_PINGREPLY) {
+      ptr2 = ptr + 48;
+      if (memcmp(ptr + 48, buf2, 4) != 0)       // from a different process?
+        return;
       pos = ptr[46];
       if (position[pos] != NULL && pos <= up_to && pos == ptr[45]) {
         if (position[pos] != NULL) {
@@ -103,40 +110,82 @@ void check_packets(u_char *foo, const struct pcap_pkthdr *header, const unsigned
         position[pos + 1] = NULL;
         finaldst = 1;
       }
-    }
-    if (ptr[40] == ICMP6_TTLEXEED && ptr[41] == 0 && len >= 100) {
-      if (type == 0) {
-        pos = ptr[94 + 8*(do_alert + do_frag + do_dst)];
-        pos2 = ptr[93 + 8*(do_alert + do_frag + do_dst)];
+    } else {
+      // now for the error icmp types
+      if (type == 0)
+        ptr2 = ptr + 96 + 8 * (add);
+      else
+        ptr2 = ptr + 104 + 0x30 + 8 * (add);
+      if (memcmp(ptr2, buf2, 4) != 0)   // from a different process?
+        return;
+      if (ptr[40] == ICMP6_TTLEXEED && ptr[41] == 0 && len >= 100) {
+        if (type == 0) {
+          pos = ptr[94 + 8 * (add)];
+          pos2 = ptr[93 + 8 * (add)];
+          if (pos != pos2 || pos > up_to) {
+            pos = ptr[94 + 8 * (add2)];
+            pos2 = ptr[93 + 8 * (add2)];
+          }
 //printf("Exceed from %s\n", thc_ipv62notation(ptr + 8));
-//printf(" POS1 is : 94 + 8*(do_alert + do_frag + do_dst) = [%d] == %d <= %d ?\n", pos, up_to);
-//printf(" POS2 is : 93 + 8*(do_alert + do_frag + do_dst) = [%d] == %d <= %d ?\n", pos2, up_to);
-      } else {
-        pos = ptr[101 + 0x30 + 8*(do_alert + do_frag + do_dst + do_toobig - 1)];
-        pos2 = ptr[100 + 0x30 + 8*(do_alert + do_frag + do_dst + do_toobig - 1)];
-      }
-      if (pos == pos2 && pos <= up_to)
-        position[pos] = thc_ipv62notation(ptr + 8);
-    }
-    if (ptr[6] == NXT_ICMP6 && ptr[40] == ICMP6_UNREACH) {
-      if (type == 0) {
-        pos = ptr[94 + 8*(do_alert + do_frag + do_dst)];
-        pos2 = ptr[93 + 8*(do_alert + do_frag + do_dst)];
-      } else {
-        pos = ptr[101 + 0x30 + 8*(do_alert + do_frag + do_dst + do_toobig - 1)];
-        pos2 = ptr[100 + 0x30 + 8*(do_alert + do_frag + do_dst + do_toobig - 1)];
-      }
-      if (pos == pos2 && pos <= up_to) {
-        if (position[pos] != NULL) {
-          position[pos] = thc_ipv62notation(ptr + 8);
-          remark[pos] = strdup("\t[unreachable message received]");
+//printf(" POS1 is : 94 + 8*(add) = [%d] <= %d ?\n", pos, up_to);
+//printf(" POS2 is : 93 + 8*(add) = [%d] <= %d ?\n", pos2, up_to);
+        } else {
+          pos = ptr[101 + 0x30 + 8 * (add + do_toobig - 1)];
+          pos2 = ptr[100 + 0x30 + 8 * (add + do_toobig - 1)];
         }
+//printf("%d == %d < %d\n", pos, pos2, up_to);
+        if (pos == pos2 && pos <= up_to)
+          position[pos] = thc_ipv62notation(ptr + 8);
+      }
+      if (ptr[40] == ICMP6_UNREACH) {
+        if (type == 0) {
+          pos = ptr[94 + 8 * (add)];
+          pos2 = ptr[93 + 8 * (add)];
+          if (pos != pos2 || pos > up_to) {
+            pos = ptr[94 + 8 * (add2)];
+            pos2 = ptr[93 + 8 * (add2)];
+          }
+        } else {
+          pos = ptr[101 + 0x30 + 8 * (add + do_toobig - 1)];
+          pos2 = ptr[100 + 0x30 + 8 * (add + do_toobig - 1)];
+        }
+        if (pos == pos2 && pos <= up_to) {
+          if (position[pos] != NULL) {
+            position[pos] = thc_ipv62notation(ptr + 8);
+            remark[pos] = strdup("\t[unreachable message received]");
+          }
 //        if (position[pos + 1][0] == '?')
           position[pos + 1] = NULL;
+        }
       }
+      if (do_dst2 == 1 && ptr[40] == ICMP6_PARAMPROB) {
+        if (type == 0) {
+          pos = ptr[94 + 8 * (add)];
+          pos2 = ptr[93 + 8 * (add)];
+          if (pos != pos2 || pos > up_to) {
+            pos = ptr[94 + 8 * (add2)];
+            pos2 = ptr[93 + 8 * (add2)];
+          }
+        } else {
+          pos = ptr[101 + 0x30 + 8 * (add + do_toobig - 1)];
+          pos2 = ptr[100 + 0x30 + 8 * (add + do_toobig - 1)];
+        }
+//thc_dump_data(ptr, len, "pkt");
+//printf("type: %d, pos %d, pos2 %d\n", type, pos, pos2);
+        if (pos == pos2 && pos <= up_to) {
+          if (position[pos] != NULL) {
+            position[pos] = thc_ipv62notation(ptr + 8);
+            remark[pos] = strdup("\t[parameter problem received]");
+          }
+//        if (position[pos + 1][0] == '?')
+          position[pos + 1] = NULL;
+          finaldst = 1;
+        }
+      }
+      
     }
   } else {
-    if (ptr[6] != NXT_ICMP6 && ptr[6] != NXT_TCP)
+    if (ptr[6] != NXT_ICMP6 && ((udp == 0 && ptr[6] != NXT_TCP) || (udp == 1 && ptr[6] != NXT_UDP)))
       return;
     if (ptr[6] == NXT_TCP) {
       si = (unsigned short int *) &ptr[42];
@@ -161,9 +210,19 @@ void check_packets(u_char *foo, const struct pcap_pkthdr *header, const unsigned
         finaldst = 1;
       }
     }
+    if (ptr[6] == NXT_UDP) {
+      si = (unsigned short int *) &ptr[42];
+      pos = htons((*si % 65536)) - baseport;
+      if (position[pos] != NULL && pos <= up_to) {
+        position[pos] = thc_ipv62notation(ptr + 8);
+        remark[pos] = strdup("\t[UDP reply received]");
+        position[pos + 1] = NULL;
+        finaldst = 1;
+      }
+    }
     if (ptr[6] == NXT_ICMP6 && ptr[40] == ICMP6_TTLEXEED && ptr[41] == 0 && len >= 100) {
-      si = (unsigned short int *) &ptr[88 + 8*(do_alert + do_frag + do_dst)];
-      ui = (unsigned int *) &ptr[92 + 8*(do_alert + do_frag + do_dst)];
+      si = (unsigned short int *) &ptr[88 + 8 * (add)];
+      ui = (unsigned int *) &ptr[92 + 8 * (add)];
       pos = htons((*si % 65536)) - baseport;
       if (pid != htonl(*ui))
         return;
@@ -171,18 +230,23 @@ void check_packets(u_char *foo, const struct pcap_pkthdr *header, const unsigned
         position[pos] = thc_ipv62notation(ptr + 8);
     }
     if (ptr[6] == NXT_ICMP6 && ptr[40] == ICMP6_UNREACH && len >= 100) {
-      si = (unsigned short int *) &ptr[88 + 8*(do_alert + do_frag + do_dst)];
-      ui = (unsigned int *) &ptr[92 + 8*(do_alert + do_frag + do_dst)];
+      si = (unsigned short int *) &ptr[88 + 8 * (add)];
+      ui = (unsigned int *) &ptr[92 + 8 * (add)];
       pos = htons((*si % 65536)) - baseport;
       if (pid != htonl(*ui))
         return;
       if (pos <= up_to) {
         if (position[pos] != NULL) {
           position[pos] = thc_ipv62notation(ptr + 8);
-          remark[pos] = strdup("\t[unreachable message received]");
+          if (udp && ptr[41] == 4) {
+            remark[pos] = strdup("\t[port unreachable message received]");
+            position[pos + 1] = NULL;
+            finaldst = 1;
+          } else
+            remark[pos] = strdup("\t[unreachable message received]");
         }
         //if (position[pos + 1][0] == '?')
-          position[pos + 1] = NULL;
+        position[pos + 1] = NULL;
       }
     }
   }
@@ -208,11 +272,14 @@ int main(int argc, char *argv[]) {
   if (argc < 3 || strncmp(argv[1], "-h", 2) == 0)
     help(argv[0]);
 
-  while ((i = getopt(argc, argv, "abdtrs:FD")) >= 0) {
+  while ((i = getopt(argc, argv, "abBdtrus:FDEX")) >= 0) {
     switch (i) {
     case 'd':
       resolve = 1;
       break;
+    case 'X':
+      debug = 1;
+      break;
     case 'a':
       do_alert = 1;
       break;
@@ -220,6 +287,10 @@ int main(int argc, char *argv[]) {
       do_toobig++;
       type = 2;
       break;
+    case 'B':
+      do_reply++;
+      type = 2;
+      break;
     case 'r':
       thc_ipv6_rawmode(1);
       rawmode = 1;
@@ -227,12 +298,19 @@ int main(int argc, char *argv[]) {
     case 't':
       tunnel = 1;
       break;
+    case 'u':
+      udp = 1;
+      break;
     case 's':
       src6 = thc_resolve6(optarg);
       break;
     case 'F':
       do_frag++;
       break;
+    case 'E':
+      do_dst++;
+      do_dst2 = 1;
+      break;
     case 'D':
       do_dst++;
       break;
@@ -257,6 +335,8 @@ int main(int argc, char *argv[]) {
     src6 = thc_get_own_ipv6(interface, dst6, prefer);
   srcmac = thc_get_own_mac(interface);
   up_to = MAX_SEND;
+  if (do_reply)
+    do_toobig = 0;
 
   if (argc - optind >= 3 && argv[optind + 2] != NULL) {
     if (type) {
@@ -266,7 +346,7 @@ int main(int argc, char *argv[]) {
     type = 1;
     dport = atoi(argv[optind + 2]);
     if (dport < 0 || dport > 65535) {
-      fprintf(stderr, "Error: tcp port (3rd option) is invalid: %s\n", argv[optind + 2]);
+      fprintf(stderr, "Error: port (3rd option) is invalid: %s\n", argv[optind + 2]);
       exit(-1);
     }
   }
@@ -306,7 +386,7 @@ int main(int argc, char *argv[]) {
       buf_len -= 8;
     orig_mtu = buf_len + offset;
   }
-  
+
   if (do_alert) {
     memset(buf3, 0, sizeof(buf3));
     buf3[0] = 5;
@@ -334,17 +414,22 @@ int main(int argc, char *argv[]) {
         if (position[i] != NULL && position[i][0] == '?') {
           if (type != 1)
             memset((char *) &j, i % 256, 4);
-          if ((pkt = thc_create_ipv6(interface, prefer, &pkt_len, src6, dst6, i, 0, 0, 0, 0)) == NULL)
+          if ((pkt = thc_create_ipv6_extended(interface, prefer, &pkt_len, src6, dst6, i, 0, 0, 0, 0)) == NULL)
             return -1;
           if (do_alert)
             if (thc_add_hdr_hopbyhop(pkt, &pkt_len, buf3, 6) < 0)
               return -1;
           if (type != 2) {
-            if (do_frag) for (m = 0; m < do_frag; m++)
-              if (thc_add_hdr_oneshotfragment(pkt, &pkt_len, getpid() + m) < 0)
-                return -1;
+            if (do_frag)
+              for (m = 0; m < do_frag; m++)
+                if (thc_add_hdr_oneshotfragment(pkt, &pkt_len, getpid() + m) < 0)
+                  return -1;
             if (do_dst) {
               memset(buf3, 0, 6);
+              if (do_dst2) {
+                buf3[0] = NXT_INVALID;
+                buf3[1] = 1;
+              }
               for (m = 0; m < do_dst; m++)
                 if (thc_add_hdr_dst(pkt, &pkt_len, buf3, 6) < 0)
                   return -1;
@@ -362,7 +447,7 @@ int main(int argc, char *argv[]) {
                 foo6[8] = 0;
               else
                 foo6[8] = 1;
-              if ((pkt2 = thc_create_ipv6(interface, 0, &pkt2_len, dst6, foo6, i, 0, 0, 0, 0)) == NULL)
+              if ((pkt2 = thc_create_ipv6_extended(interface, 0, &pkt2_len, dst6, foo6, i, 0, 0, 0, 0)) == NULL)
                 return -1;
               if (thc_add_icmp6(pkt2, &pkt2_len, ICMP6_PINGREQUEST, 0, j, (unsigned char *) &buf, m, 0) < 0)
                 return -1;
@@ -381,19 +466,24 @@ int main(int argc, char *argv[]) {
                 m = do_hdr_size;
               else
                 m = 14;
-              if (thc_add_icmp6(pkt, &pkt_len, ICMP6_TOOBIG, 0, 1480, (unsigned char *) ipv6->pkt + m, 1280 - 40 - 8 - 8, 0) < 0)
+              if (do_reply) {
+                if (thc_add_icmp6(pkt, &pkt_len, ICMP6_PINGREPLY, 0, 1480, (unsigned char *) ipv6->pkt + m, 1280 - 40 - 8 - 8, 0) < 0)
+                  return -1;
+              } else if (thc_add_icmp6(pkt, &pkt_len, ICMP6_TOOBIG, 0, 1480, (unsigned char *) ipv6->pkt + m, 1280 - 40 - 8 - 8, 0) < 0)
                 return -1;
               pkt2 = thc_destroy_packet(pkt2);
-            } else
-              if (thc_add_tcp(pkt, &pkt_len, baseport + i, dport, pid, 0, TCP_SYN, 5760, 0, NULL, 0, buf_len > 0 ? buf : NULL, buf_len) < 0)
+            } else if (udp) {
+              if (thc_add_udp(pkt, &pkt_len, baseport + i, dport, 0, buf_len > 0 ? buf : NULL, buf_len) < 0)
                 return -1;
+            } else if (thc_add_tcp(pkt, &pkt_len, baseport + i, dport, pid, 0, TCP_SYN, 5760, 0, NULL, 0, buf_len > 0 ? buf : NULL, buf_len) < 0)
+              return -1;
           }
           if (thc_generate_and_send_pkt(interface, srcmac, mac, pkt, &pkt_len) < 0) {
             fprintf(stderr, "Error: Can not send packet, exiting ...\n");
             exit(-1);
           }
           pkt = thc_destroy_packet(pkt);
-          usleep(200);
+          usleep(1000);
         } else if (position[i] == NULL)
           up_to = i - 1;
       }
@@ -403,7 +493,7 @@ int main(int argc, char *argv[]) {
     while (passed + k >= time(NULL) && complete == 0)
       thc_pcap_check(p, (char *) check_packets, NULL);
 
-    if (complete == 0 && finaldst == 0 && k + 1 < SENDS && up_to >= MAX_SEND && position[up_to] != NULL /*&& position[up_to][0] != '?'*/) {
+    if (complete == 0 && finaldst == 0 && k + 1 < SENDS && up_to >= MAX_SEND && position[up_to] != NULL /*&& position[up_to][0] != '?' */ ) {
       if (debug)
         printf("DEBUG: increasing range from %d to %d\n", up_to, up_to + INCREASE);
       up_to += INCREASE;
@@ -473,7 +563,7 @@ int main(int argc, char *argv[]) {
     printf("\n");
   }
   if (notreached) {
-    if (do_toobig) 
+    if (do_toobig)
       printf("With the -b TooBig option, the destination will not send a reply\n");
     else
       printf("The destination seems to be filtered.\n");
